@@ -17,18 +17,24 @@
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 
 
-import base64
 import binascii
 from Crypto.Hash import SHA, HMAC
-import UserDict
+from paramiko.common import rng
+from paramiko.py3compat import b, u, encodebytes, decodebytes
+
+try:
+    from collections import MutableMapping
+except ImportError:
+    # noinspection PyUnresolvedReferences
+    from UserDict import DictMixin as MutableMapping
 
-from paramiko.common import *
 from paramiko.dsskey import DSSKey
 from paramiko.rsakey import RSAKey
 from paramiko.util import get_logger, constant_time_bytes_eq
+from paramiko.ecdsakey import ECDSAKey
 
 
-class HostKeys (UserDict.DictMixin):
+class HostKeys (MutableMapping):
     """
     Representation of an OpenSSH-style "known hosts" file.  Host keys can be
     read from one or more files, and then individual hosts can be looked up to
@@ -82,20 +88,19 @@ class HostKeys (UserDict.DictMixin):
 
         :raises IOError: if there was an error reading the file
         """
-        f = open(filename, 'r')
-        for lineno, line in enumerate(f):
-            line = line.strip()
-            if (len(line) == 0) or (line[0] == '#'):
-                continue
-            e = HostKeyEntry.from_line(line, lineno)
-            if e is not None:
-                _hostnames = e.hostnames
-                for h in _hostnames:
-                    if self.check(h, e.key):
-                        e.hostnames.remove(h)
-                if len(e.hostnames):
-                    self._entries.append(e)
-        f.close()
+        with open(filename, 'r') as f:
+            for lineno, line in enumerate(f):
+                line = line.strip()
+                if (len(line) == 0) or (line[0] == '#'):
+                    continue
+                e = HostKeyEntry.from_line(line, lineno)
+                if e is not None:
+                    _hostnames = e.hostnames
+                    for h in _hostnames:
+                        if self.check(h, e.key):
+                            e.hostnames.remove(h)
+                    if len(e.hostnames):
+                        self._entries.append(e)
 
     def save(self, filename):
         """
@@ -110,12 +115,11 @@ class HostKeys (UserDict.DictMixin):
 
         .. versionadded:: 1.6.1
         """
-        f = open(filename, 'w')
-        for e in self._entries:
-            line = e.to_line()
-            if line:
-                f.write(line)
-        f.close()
+        with open(filename, 'w') as f:
+            for e in self._entries:
+                line = e.to_line()
+                if line:
+                    f.write(line)
 
     def lookup(self, hostname):
         """
@@ -126,12 +130,26 @@ class HostKeys (UserDict.DictMixin):
         :param str hostname: the hostname (or IP) to lookup
         :return: dict of `str` -> `.PKey` keys associated with this host (or ``None``)
         """
-        class SubDict (UserDict.DictMixin):
+        class SubDict (MutableMapping):
             def __init__(self, hostname, entries, hostkeys):
                 self._hostname = hostname
                 self._entries = entries
                 self._hostkeys = hostkeys
 
+            def __iter__(self):
+                for k in self.keys():
+                    yield k
+
+            def __len__(self):
+                return len(self.keys())
+
+            def __delitem__(self, key):
+                for e in list(self._entries):
+                    if e.key.get_name() == key:
+                        self._entries.remove(e)
+                else:
+                    raise KeyError(key)
+
             def __getitem__(self, key):
                 for e in self._entries:
                     if e.key.get_name() == key:
@@ -158,7 +176,7 @@ class HostKeys (UserDict.DictMixin):
         entries = []
         for e in self._entries:
             for h in e.hostnames:
-                if h.startswith('|1|') and constant_time_bytes_eq(self.hash_host(hostname, h), h) or h == hostname:
+                if h.startswith('|1|') and not hostname.startswith('|1|') and constant_time_bytes_eq(self.hash_host(hostname, h), h) or h == hostname:
                     entries.append(e)
         if len(entries) == 0:
             return None
@@ -180,7 +198,7 @@ class HostKeys (UserDict.DictMixin):
         host_key = k.get(key.get_name(), None)
         if host_key is None:
             return False
-        return str(host_key) == str(key)
+        return host_key.asbytes() == key.asbytes()
 
     def clear(self):
         """
@@ -188,6 +206,16 @@ class HostKeys (UserDict.DictMixin):
         """
         self._entries = []
 
+    def __iter__(self):
+        for k in self.keys():
+            yield k
+
+    def __len__(self):
+        return len(self.keys())
+
+    def __delitem__(self, key):
+        k = self[key]
+
     def __getitem__(self, key):
         ret = self.lookup(key)
         if ret is None:
@@ -238,10 +266,10 @@ class HostKeys (UserDict.DictMixin):
         else:
             if salt.startswith('|1|'):
                 salt = salt.split('|')[2]
-            salt = base64.decodestring(salt)
+            salt = decodebytes(b(salt))
         assert len(salt) == SHA.digest_size
-        hmac = HMAC.HMAC(salt, hostname, SHA).digest()
-        hostkey = '|1|%s|%s' % (base64.encodestring(salt), base64.encodestring(hmac))
+        hmac = HMAC.HMAC(salt, b(hostname), SHA).digest()
+        hostkey = '|1|%s|%s' % (u(encodebytes(salt)), u(encodebytes(hmac)))
         return hostkey.replace('\n', '')
     hash_host = staticmethod(hash_host)
 
@@ -290,14 +318,18 @@ class HostKeyEntry:
         # Decide what kind of key we're looking at and create an object
         # to hold it accordingly.
         try:
+            key = b(key)
             if keytype == 'ssh-rsa':
-                key = RSAKey(data=base64.decodestring(key))
+                key = RSAKey(data=decodebytes(key))
             elif keytype == 'ssh-dss':
-                key = DSSKey(data=base64.decodestring(key))
+                key = DSSKey(data=decodebytes(key))
+            elif keytype == 'ecdsa-sha2-nistp256':
+                key = ECDSAKey(data=decodebytes(key))
             else:
                 log.info("Unable to handle key of type %s" % (keytype,))
                 return None
-        except binascii.Error, e:
+
+        except binascii.Error as e:
             raise InvalidHostKey(line, e)
 
         return cls(names, key)
