@@ -232,7 +232,7 @@ class Transport (threading.Thread):
         self._channels = ChannelMap()
         self.channel_events = {}       # (id -> Event)
         self.channels_seen = {}        # (id -> True)
-        self._channel_counter = 1
+        self._channel_counter = 0
         self.window_size = 65536
         self.max_packet_size = 34816
         self._forward_agent_handler = None
@@ -963,10 +963,11 @@ class Transport (threading.Thread):
     def get_banner(self):
         """
         Return the banner supplied by the server upon connect. If no banner is
-        supplied, this method returns C{None}.
+        supplied, this method returns ``None``.
 
-        @return: server supplied banner, or C{None}.
-        @rtype: string
+        :returns: server supplied banner (`str`), or ``None``.
+
+        .. versionadded:: 1.13
         """
         if not self.active or (self.auth_handler is None):
             return None
@@ -1986,7 +1987,7 @@ class Transport (threading.Thread):
         always_display = m.get_boolean()
         msg = m.get_string()
         lang = m.get_string()
-        self._log(DEBUG, 'Debug msg: ' + util.safe_string(msg))
+        self._log(DEBUG, 'Debug msg: {0}'.format(util.safe_string(msg)))
 
     def _get_subsystem_handler(self, name):
         try:
